@@ -170,8 +170,7 @@ gyrus_admin_init_get_widgets (GyrusAdmin *admin)
 	gchar *file_glade;
 	GtkWidget *table;
 
-	file_glade = g_strdup_printf ("%s%c%s", GYRUS_GLADE_DIR,
-				      G_DIR_SEPARATOR, "page.glade");
+	file_glade = g_build_filename (GYRUS_GLADE_DIR, "page.glade", NULL);
 	xml_page = glade_xml_new (file_glade, "table_page", "gyrus");
 	
 	g_free (file_glade);
