@@ -49,56 +49,6 @@ struct _GyrusAdmin {
 
 struct _GyrusAdminClass {
 	 GtkVBoxClass vbox_class;
-	int connected_signal_id;
-	int disconnected_signal_id;
-	int acl_entry_selection_changed;
-	int mailbox_selection_changed;
-};
-
-/*** Gyrus Administrator Private Data ***/
-struct _GyrusAdminPrivate {
-	GtkWidget *label_host;
-	GtkWidget *label_user;
-	GtkWidget *label_port;
-/*	GtkWidget *entry_pass; */
-	GtkWidget *button_connect;
-
-	GtkWidget *label_mailbox_owner;
-	GtkWidget *label_quota_limit;
-	GtkWidget *label_quota_free;
-	GtkWidget *entry_quota_new;
-	GtkWidget *button_quota_apply;
-	
-	GtkWidget *entry_find_account;
-	GtkWidget *window_find_account;
-	
-	GtkWidget *button_acl_apply;
-	GtkWidget *button_acl_add;
-	GtkWidget *button_acl_remove;
-	GtkWidget *button_close_page;
-	
-	GtkWidget *button_user_create;
-	GtkWidget *button_user_remove;
-       	
-	GtkWidget *label_mailbox_name;
-	GtkWidget *label_page;
-
-	GtkWidget *expander_modify_quota;
-	GtkWidget *expander_modify_acl;
-	
-	GtkTreeView *treeview_users;
-	GtkTreeView *treeview_acl;
-	GtkTreeView *treeview_corrupt_mailbox;
-
-	GtkWidget *label_acl;
-	GtkWidget *scrolled_acl;
-	
-	GIOChannel *iochannel;
-	GTcpSocket *tcp_socket;
-
-	GyrusSession *session;
-
-	gpointer *parent_app;
 };
 
 typedef enum {
@@ -152,15 +102,6 @@ GType gyrus_admin_get_type (void);
 */
 GtkWidget * gyrus_admin_new (GyrusSession *session);
 
-/**
-   Sets the parent application of the #GyrusAdmin instance.
-
-   It is needed that any #GyrusAdmin had a parent application,
-   so it can set sensitive/insensitive widgets on it when needed.
-
-*/
-void gyrus_admin_set_parent_app (GyrusAdmin *admin, gpointer app);
-
 /**
    Gets the @host where the #GyrusAdmin is connected.
 */
@@ -244,5 +185,11 @@ GIOError gyrus_admin_write_channel (GyrusAdmin *admin, gchar *message);
 gchar *
 gyrus_admin_get_mailbox_from_list_message (gchar *msg);
 
+/**
+   Returns the treeview with the mailboxes. You need to
+   unref it once you are done with it.
+*/
+GtkTreeView * gyrus_admin_get_users_treeview (GyrusAdmin *admin);
+
 #endif /* GYRUS_ADMIN_H */
 
