@@ -72,8 +72,7 @@ gyrus_dialog_password_new (void)
 	gchar *password = NULL;
 	gint result;
 
-	file_glade = g_strdup_printf ("%s%c%s", GYRUS_GLADE_DIR,
-				      G_DIR_SEPARATOR, "page.glade");
+	file_glade = g_build_filename (GYRUS_GLADE_DIR, "page.glade", NULL);
 	xml = glade_xml_new (file_glade, "dialog_password", "gyrus");
 	g_free (file_glade);
 	
