@@ -35,8 +35,7 @@ gyrus_preferences_show_dialog (void)
        	gchar *file_glade;
 	GtkWidget *window_preferences;
 
-	file_glade = g_strdup_printf ("%s%c%s", GYRUS_GLADE_DIR,
-				      G_DIR_SEPARATOR, "preferences.glade");
+	file_glade = g_build_filename (GYRUS_GLADE_DIR, "preferences.glade", NULL);
 	xml = glade_xml_new (file_glade, NULL, NULL);
 
 	window_preferences = glade_xml_get_widget (xml, "preferences");
