@@ -712,8 +712,7 @@ gyrus_session_load_interface (gpointer user_data)
 	GladeXML *xml;
 	gchar *file_glade;
         
-	file_glade = g_strdup_printf ("%s%c%s", GYRUS_GLADE_DIR,
-				      G_DIR_SEPARATOR, "sessions.glade");
+	file_glade = g_build_filename (GYRUS_GLADE_DIR, "sessions.glade", NULL);
 	xml = glade_xml_new (file_glade, "dialog_sessions", "gyrus");
 	g_free (file_glade);
 	
@@ -771,8 +770,7 @@ gyrus_session_edit_load_interface (gpointer user_data)
 	GladeXML *xml;
 	gchar *file_glade;
         
-	file_glade = g_strdup_printf ("%s%c%s", GYRUS_GLADE_DIR,
-				      G_DIR_SEPARATOR, "sessions.glade");
+	file_glade = g_build_filename (GYRUS_GLADE_DIR, "sessions.glade", NULL);
 	xml = glade_xml_new (file_glade, "dialog_session_edit", "gyrus");
 	g_free (file_glade);
 	
